import stan
import matplotlib.pyplot as plt
import numpy as np
import arviz as az
import seaborn as sb

RawData = [[1,  1, 0, 0, 1, 0, 0],
           [1,  1, 0, 0, 0, 1, 0],
           [10, 1, 0, 0, 0, 0, 1],
           [3,  0, 1, 0, 1, 0, 0],
           [2,  0, 1, 0, 0, 1, 0],
           [10, 0, 1, 0, 0, 0, 1],
           [6,  0, 0, 1, 1, 0, 0],
           [2,  0, 0, 1, 0, 1, 0],
           [4,  0, 0, 1, 0, 0, 1]]

F = []
Xr1 = []
Xr2 = []
Xr3 = []
Xc1 = []
Xc2 = []
Xc3 = []
for v in RawData:
    F.append(v[0])
    Xr1.append(v[1])
    Xr2.append(v[2])
    Xr3.append(v[3])
    Xc1.append(v[4])
    Xc2.append(v[5])
    Xc3.append(v[6])

for i in range(9):
    print(' ', F[i], '  ', Xr1[i], '  ', Xr2[i], '  ', Xr3[i],
          '  ', Xc1[i], '  ', Xc2[i], '  ', Xc3[i])

N = 9
Data = {'N': N, 'F': F, 'Xr1': Xr1, 'Xr2': Xr2, 'Xr3': Xr3,
        'Xc1': Xc1, 'Xc2': Xc2, 'Xc3': Xc3}
with open('Poisson.stan', 'r') as f_stan:
    sm = stan.build(f_stan.read(), data = Data)
fit = sm.sample()

f = open('Results.txt', 'w')

i_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(i_data)
print('Summary =\n',smry)
f.write('\nSummary...\n')
f.write(smry.__str__())
f.write('\n')

d_fm = fit.to_frame()

az.plot_trace(i_data, var_names = ('a2', 'a3', 'b2', 'b3'))
plt.show()

a3 = d_fm['a3']
cnt_a3_pos = (a3 > 0.0).sum()
print('\nP(a3 > 0.0) = {0:.3f}'.format(cnt_a3_pos / len(a3)))
f.write('\nP(a3 > 0.0) = {0:.3f}\n'.format(cnt_a3_pos / len(a3)))


b3 = d_fm['b3']
cnt_b3_pos = (b3 > 0.0).sum()
print('P(b3 > 0.0) = {0:.3f}'.format(cnt_b3_pos / len(b3)))
f.write('\nP(b3 > 0.0) = {0:.3f}\n'.format(cnt_b3_pos / len(b3)))

sb.kdeplot(d_fm['a2'], label = 'a2')
sb.kdeplot(d_fm['a3'], label = 'a3')
sb.kdeplot(d_fm['b2'], label = 'b2')
sb.kdeplot(d_fm['b3'], label = 'b3')
plt.legend()
plt.show()

g33 = d_fm['g33']
print(len(g33))
print(np.shape(g33))
cnt_g33_neg = (g33 < 0.0).sum()
print('P(g33 < 0.0) = {0:.3f}'.format(cnt_g33_neg / len(g33)))
f.write('\nP(g33 < 0.0) = {0:.3f}\n'.format(cnt_g33_neg / len(g33)))

sb.kdeplot(d_fm['g22'], label = 'g22')
sb.kdeplot(d_fm['g23'], label = 'g23')
sb.kdeplot(d_fm['g32'], label = 'g32')
sb.kdeplot(d_fm['g33'], label = 'g33')
plt.legend()
plt.show()

lmbd_hat = np.empty(N)
for i in range(N):
    lmbd_hat[i] = np.median(d_fm[f'Lmbd.{i+1}'])
    plt.plot(lmbd_hat[i], F[i], 'o', c='b')
min_lmbd_hat = np.min(lmbd_hat)
max_lmbd_hat = np.max(lmbd_hat)
plt.plot([min_lmbd_hat, max_lmbd_hat], [min_lmbd_hat, max_lmbd_hat],
         label = '$F = \hat{\lambda}$')
plt.xlabel('$\hat{\lambda}$', fontsize = 16)
plt.ylabel('F', fontsize = 14)
plt.legend(fontsize = 16)
plt.tight_layout()
plt.show()

f.close()
print('Results.txt was saved.')

